package hilfsklassen;

import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 


import java.util.ArrayList;

/**
 * Linien-Objekte repräsentieren Sammlungen von Strecken, die
 * zunächst in ihrem Körper eigenen Koordinatensystem definiert werden:
 * linienKoerper
 * 
 * die anderen Repräsentationen dienen dazu, die Linien in einem Raumkoordinatensystem
 * zu transformieren:
 * linienInert
 * transformToInert()
 * 
 * ...und es als Pixel-Liste auf einem Bildschirm sichtbar zu machen:
 * linienPixel
 * transformToPixel()
 * draw()
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Linien
{
    private PApplet pap;

    public ArrayList<double[]> linienKoerper = new ArrayList<double[]>();
    public ArrayList<double[]> linienInert = new ArrayList<double[]>();
    public ArrayList<int[]> linienPixel = new ArrayList<int[]>();
    
    public Linien(PApplet pap)
    {
        this.pap = pap;
    }
    
    /**
     * Linie im Körpereigenen-Koordinatensystem mit Start- und Endpunkt ergänzen.
     */
    public void add(double x1, double y1, double x2, double y2)
    {
        linienKoerper.add(new double[] {x1,y1,x2,y2});
        linienInert.add(new double[] {x1,y1,x2,y2}); //dummy
        linienPixel.add(new int[] {(int)x1,(int)y1,(int)x2,(int)y2});     //dummy
    }
    
    /**
     * aktualisiert die Transformation ins Raum-Koordinatensystem gemäß den 
     * Übergabeparametern.
     * x, y translatorische Verschiebung
     * phi Drehung des Körpers bzgl. des Inertialkoordinatensystems
     */
    public void transformToInert(double x, double y, double phi)
    {
        for(int i=0;i<linienKoerper.size();i++)
        {
            double[] u = linienKoerper.get(i);
            double[] v = linienInert.get(i);
            //  cos -sin
            //  sin  cos
            v[0] = Math.cos(phi)*u[0] - Math.sin(phi)*u[1];
            v[1] = Math.sin(phi)*u[0] + Math.cos(phi)*u[1];
            
            v[0]+=x;
            v[1]+=y;
            
            v[2] = Math.cos(phi)*u[2] - Math.sin(phi)*u[3];
            v[3] = Math.sin(phi)*u[2] + Math.cos(phi)*u[3];
            
            v[2]+=x;
            v[3]+=y;
            
        }
    }
    
    /**
     * aktualisiert die Transformation in Pixel-Koordinaten
     * Achtung: transformToInert() muss vorher ausgeführt worden sein!
     */
    public void transformToPixel(double skalierung, int XOFF, int YOFF, int BREITE, int HOEHE)
    {
        for(int i=0;i<linienInert.size();i++)
        {
            double[] u = linienInert.get(i);
            int[] v = linienPixel.get(i);
            v[0] = (int)(u[0]*skalierung)+XOFF;
            v[1] = HOEHE - ((int)(u[1]*skalierung)+YOFF);
            v[2] = (int)(u[2]*skalierung)+XOFF;
            v[3] = HOEHE - ((int)(u[3]*skalierung)+YOFF);
            
        }        
    }
    
    public void show()
    {
        System.out.println("Körperkoordinaten");
        System.out.println("x1 y1 x2 y2");
        for(int i=0;i<linienKoerper.size();i++)
        {
            double[] v=linienKoerper.get(i);
            System.out.print(i+".: ");
            for(int k=0;k<v.length;k++)
                System.out.print(v[k]+" ");
            System.out.println();                
        }
        System.out.println("Inertialkoordinaten");
        System.out.println("x1 y1 x2 y2");
        for(int i=0;i<linienKoerper.size();i++)
        {
            double[] v=linienInert.get(i);
            System.out.print(i+".: ");
            for(int k=0;k<v.length;k++)
                System.out.print(v[k]+" ");
            System.out.println();                
        }
        System.out.println("Bildschirmkoordinaten");
        System.out.println("x1 y1 x2 y2");
        for(int i=0;i<linienKoerper.size();i++)
        {
            int[] v=linienPixel.get(i);
            System.out.print(i+".: ");
            for(int k=0;k<v.length;k++)
                System.out.print(v[k]+" ");
            System.out.println();                
        }
        
    }
    
    public void draw(int ROT, int GRUEN, int BLAU)
    {
        pap.noFill();
        pap.stroke(ROT,GRUEN,BLAU);
        for(int i=0;i<linienPixel.size();i++)
        {
            int[] v=linienPixel.get(i);
            pap.line(v[0],v[1],v[2],v[3]);
        }
    }
}


